@@ -6,7 +6,7 @@
 #  Author: Sebastian Heinlein <sebastian.heinlein@web.de>
 #          Michael Vogt <michael.vogt@ubuntu.com>
 #
-#  This modul provides an inheritance of the gtk.TextView that is 
+#  This modul provides an inheritance of the Gtk.TextView that is 
 #  aware of http URLs and allows to open them in a browser.
 #  It is based on the pygtk-demo "hypertext".
 # 
@@ -26,18 +26,19 @@
 #  USA
 
 
-import pygtk
-import gtk
-import pango
-import subprocess
-import os
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GObject
+from gi.repository import Pango
 from gettext import gettext as _
 
-class ChangelogViewer(gtk.TextView):
+from ReleaseNotesViewer import open_url
+
+class ChangelogViewer(Gtk.TextView):
     def __init__(self, changelog=None):
-        """Init the ChangelogViewer as an Inheritance of the gtk.TextView"""
+        """Init the ChangelogViewer as an Inheritance of the Gtk.TextView"""
         # init the parent
-        gtk.TextView.__init__(self)
+        GObject.GObject.__init__(self)
         # global hovering over link state
         self.hovering = False
         self.first = True
@@ -48,7 +49,7 @@ class ChangelogViewer(gtk.TextView):
         self.set_right_margin(4)
         self.set_left_margin(4)
         self.set_pixels_above_lines(4)
-        self.buffer = gtk.TextBuffer()
+        self.buffer = Gtk.TextBuffer()
         self.set_buffer(self.buffer)
         self.connect("button-press-event", self.button_press_event)
         self.connect("motion-notify-event", self.motion_notify_event)
@@ -61,15 +62,18 @@ class ChangelogViewer(gtk.TextView):
             
     def create_context_menu(self, url):
         """Create the context menu to be displayed when links are right clicked"""
-        self.menu = gtk.Menu()
+        self.menu = Gtk.Menu()
         
         # create menu items
-        item_grey_link = gtk.MenuItem(url)
+        item_grey_link = Gtk.MenuItem()
+        item_grey_link.set_label(url)
         item_grey_link.connect("activate", self.handle_context_menu, "open", url)
-        item_seperator = gtk.MenuItem()
-        item_open_link = gtk.MenuItem(_("Open Link in Browser"))
+        item_seperator = Gtk.MenuItem()
+        item_open_link = Gtk.MenuItem()
+        item_open_link.set_label(_("Open Link in Browser"))
         item_open_link.connect("activate", self.handle_context_menu, "open", url)
-        item_copy_link = gtk.MenuItem(_("Copy Link to Clipboard"))
+        item_copy_link = Gtk.MenuItem()
+        item_copy_link.set_label(_("Copy Link to Clipboard"))
         item_copy_link.connect("activate", self.handle_context_menu, "copy", url)
         
         # add menu items
@@ -82,22 +86,27 @@ class ChangelogViewer(gtk.TextView):
     def handle_context_menu(self, menuitem, action, url):
         """Handle activate event for the links' context menu"""
         if action == "open":
-            self.open_url(url)
+            open_url(url)
         if action == "copy":
-            cb = gtk.Clipboard()
-            cb.set_text(url)
+            # the following two lines used to be enough - then gtk3/pygi
+            # came along ...
+            #cb = Gtk.Clipboard()
+            #cb.set_text(url)
+            display = Gdk.Display.get_default()
+            selection = Gdk.Atom.intern ("CLIPBOARD", False)
+            cb = Gtk.Clipboard.get_for_display(display, selection)
+            cb.set_text(url, -1)
             cb.store()
 
     def tag_link(self, start, end, url):
         """Apply the tag that marks links to the specified buffer selection"""
-        tagged = False
         tags = start.get_tags()
         for tag in tags:
             url = tag.get_data("url")
             if url != "":
                 return
         tag = self.buffer.create_tag(None, foreground="blue",
-                                     underline=pango.UNDERLINE_SINGLE)
+                                     underline=Pango.Underline.SINGLE)
         tag.set_data("url", url)
         self.buffer.apply_tag(tag , start, end)
 
@@ -137,13 +146,12 @@ class ChangelogViewer(gtk.TextView):
                        ]
         # init
         iter = buffer.get_iter_at_offset(iter_end.get_offset() - len(text))
-        iter_real_end = buffer.get_end_iter()
 
         # search for the next match in the buffer
         for (start_str, end_list, url_prefix) in search_items:
             while True:
                 ret = iter.forward_search(start_str,
-                                          gtk.TEXT_SEARCH_VISIBLE_ONLY,
+                                          Gtk.TextSearchFlags.VISIBLE_ONLY,
                                           iter_end)
                 # if we reach the end break the loop
                 if not ret:
@@ -192,7 +200,7 @@ class ChangelogViewer(gtk.TextView):
                 return False
 
         # get the iter at the mouse position
-        (x, y) = self.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET,
+        (x, y) = self.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                               int(event.x), int(event.y))
         iter = self.get_iter_at_location(x, y)
         
@@ -202,46 +210,28 @@ class ChangelogViewer(gtk.TextView):
             url = tag.get_data("url")
             if url != None:
                 if event.button == 1: 
-                    self.open_url(url)
+                    open_url(url)
                     break
                 if event.button == 3:
                     self.create_context_menu(url)
-                    self.menu.popup(None, None, None, event.button, event.time)
+                    self.menu.popup(None, None, None, None, event.button, event.time)
                     return True
 
-    def open_url(self, url):
-        """Open the specified URL in a browser"""
-        # Find an appropiate browser
-        if os.path.exists("/usr/bin/xdg-open"):
-            command = ["xdg-open", url]
-        elif os.path.exists("/usr/bin/exo-open"):
-            command = ["exo-open", url]
-        elif os.path.exists('/usr/bin/gnome-open'):
-            command = ['gnome-open', url]
-        else:
-            command = ['x-www-browser', url]
-
-        # Avoid to run the browser as user root
-        if os.getuid() == 0 and os.environ.has_key('SUDO_USER'):
-            command = ['sudo', '-u', os.environ['SUDO_USER']] + command
-
-        subprocess.Popen(command)
-
     def motion_notify_event(self, text_view, event):
         """callback for the mouse movement event, that calls the
            check_hovering method with the mouse postition coordiantes"""
-        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET,
+        x, y = text_view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                                  int(event.x), int(event.y))
         self.check_hovering(x, y)
-        self.window.get_pointer()
+        self.get_window(Gtk.TextWindowType.TEXT).get_pointer()
         return False
     
     def visibility_notify_event(self, text_view, event):
         """callback if the widgets gets visible (e.g. moves to the foreground)
            that calls the check_hovering method with the mouse position
            coordinates"""
-        (wx, wy, mod) = text_view.window.get_pointer()
-        (bx, by) = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, wx,
+        (screen, wx, wy, mod) = text_view.get_window(Gtk.TextWindowType.TEXT).get_pointer()
+        (bx, by) = text_view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET, wx,
                                                      wy)
         self.check_hovering(bx, by)
         return False
@@ -267,18 +257,18 @@ class ChangelogViewer(gtk.TextView):
             self.hovering = _hovering
             # Set the appropriate cursur icon
             if self.hovering:
-                self.get_window(gtk.TEXT_WINDOW_TEXT).\
-                        set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
+                self.get_window(Gtk.TextWindowType.TEXT).\
+                        set_cursor(Gdk.Cursor.new(Gdk.CursorType.HAND2))
             else:
-                self.get_window(gtk.TEXT_WINDOW_TEXT).\
-                        set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))
+                self.get_window(Gtk.TextWindowType.TEXT).\
+                        set_cursor(Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR))
 
 
 if __name__ == "__main__":
-    w = gtk.Window()
+    w = Gtk.Window()
     cv = ChangelogViewer()
     changes = cv.get_buffer()
-    changes.create_tag("versiontag", weight=pango.WEIGHT_BOLD)
+    changes.create_tag("versiontag", weight=Pango.Weight.BOLD)
     changes.set_text("""
 
 Version 6-14-0ubuntu1.9.04:
@@ -290,4 +280,4 @@ Version 6-14-0ubuntu1.9.04:
 
     w.add(cv)
     w.show_all()
-    gtk.main()
+    Gtk.main()
